package com.mycroftmind.samples.win.asynctest;


import com.mycroftmind.samples.win.asynctest.model.SampleEntity;
import java.util.List;
import java.util.concurrent.Future;
import javax.ejb.AsyncResult;
import javax.ejb.Asynchronous;
import javax.ejb.EJB;
import javax.ejb.LocalBean;
import javax.ejb.Lock;
import javax.ejb.LockType;
import javax.ejb.Singleton;



/**
 * The EJB for testing asynchronous method invocation with a transaction.
 *
 * @author win
 */
@Singleton
@LocalBean
public class AsyncTestBean {

    /**
     * The delay in milliseconds.
     */
    private static final long DELAY = 60000;

    /**
     * The provider of database access for SampleEntity instances.
     */
    @EJB
    private AsyncTestCRUDBean crudBean;

    /**
     * This method waits for #DELAY milliseconds and then persists a new SampleEntity instance.
     * It should cause an exception during transaction commit to be thrown.
     *
     * @return string "test"
     */
    @Asynchronous
    @Lock(LockType.READ)
    public Future<String> testMethod() {
        System.out.println("Asynchronous method started.");

        final SampleEntity sampleEntity = new SampleEntity();
        // emulate some lenghty processing
        try {
            Thread.sleep(DELAY);
        } catch (InterruptedException ex) {
            ex.printStackTrace();
        }

        this.crudBean.persist(sampleEntity);

        System.out.println("Asynchronous method done.");
        return new AsyncResult<String>("test");
    }

    /**
     * Returns all records stored in the database.
     *
     * @return the list of SampleEntity instances
     */
    @Lock(LockType.READ)
    public List<SampleEntity> getSamples() {
        return this.crudBean.getSamples();
    }

}
